@@ -1,7 +1,9 @@
 # Copyright 2012 by Wibowo Arindrarto.  All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 
 """Bio.SearchIO parser for BLAST+ tab output format, with or without comments."""
 
@@ -67,10 +69,17 @@ _LONG_SHORT_MAP = {
     '% hsp coverage': 'qcovhsp',
 }
 
+
 # function to create a list from semicolon-delimited string
 # used in BlastTabParser._parse_result_row
-_list_semicol = lambda x: x.split(';')
-_list_diamond = lambda x: x.split('<>')
+def _list_semicol(s):
+    return s.split(';')
+
+
+def _list_diamond(s):
+    return s.split('<>')
+
+
 # column to class attribute map
 _COLUMN_QRESULT = {
     'qseqid': ('id', str),
@@ -131,11 +140,12 @@ _SUPPORTED_FIELDS = set(list(_COLUMN_QRESULT) + list(_COLUMN_HIT) +
 # column order in the non-commented tabular output variant
 # values must be keys inside the column-attribute maps above
 _DEFAULT_FIELDS = ['qseqid', 'sseqid', 'pident', 'length', 'mismatch',
-        'gapopen', 'qstart', 'qend', 'sstart', 'send', 'evalue', 'bitscore']
+                   'gapopen', 'qstart', 'qend', 'sstart', 'send',
+                   'evalue', 'bitscore']
 # one field from each of the following sets must exist in order for the
 # parser to work
-_MIN_QUERY_FIELDS = set(['qseqid', 'qacc', 'qaccver'])
-_MIN_HIT_FIELDS = set(['sseqid', 'sacc', 'saccver', 'sallseqid'])
+_MIN_QUERY_FIELDS = {'qseqid', 'qacc', 'qaccver'}
+_MIN_HIT_FIELDS = {'sseqid', 'sacc', 'saccver', 'sallseqid'}
 
 # simple function to create BLAST HSP attributes that may be computed if
 # other certain attributes are present
@@ -155,47 +165,35 @@ def _compute_gapopen_num(hsp):
 
 def _augment_blast_hsp(hsp, attr):
     """Calculate the given HSP attribute, for writing (PRIVATE)."""
-    if attr == 'aln_span':
-        # aln_span is number of identical matches + mismatches + gaps
-        func = lambda hsp: hsp.ident_num + hsp.mismatch_num + hsp.gap_num
-    # ident and gap will require the num values to be computed first
-    elif attr.startswith('ident'):
-        func = lambda hsp: hsp.aln_span - hsp.mismatch_num - hsp.gap_num
-    elif attr.startswith('gap'):
-        func = lambda hsp: hsp.aln_span - hsp.ident_num - hsp.mismatch_num
-    elif attr == 'mismatch_num':
-        func = lambda hsp: hsp.aln_span - hsp.ident_num - hsp.gap_num
-    elif attr == 'gapopen_num':
-        if not hasattr(hsp, 'query') or not hasattr(hsp, 'hit'):
-            # mock function so that the except clause below is triggered
-            # as both the query and hit are required to compute gapopen
-            def mock(hsp):
-                raise AttributeError
-            func = mock
-        else:
-            func = _compute_gapopen_num
-
-    # set the num values
-    # requires the endswith check, since we only want to set 'num' or 'span'
-    # attributes here
     if not hasattr(hsp, attr) and not attr.endswith('_pct'):
-        value = func(hsp)
-        setattr(hsp, attr, value)
+
+        # aln_span is number of identical matches + mismatches + gaps
+        if attr == 'aln_span':
+            hsp.aln_span = hsp.ident_num + hsp.mismatch_num + hsp.gap_num
+
+        # ident and gap requires the num values to be computed first
+        elif attr.startswith('ident'):
+            setattr(hsp, attr, hsp.aln_span - hsp.mismatch_num - hsp.gap_num)
+        elif attr.startswith('gap'):
+            setattr(hsp, attr, hsp.aln_span - hsp.ident_num - hsp.mismatch_num)
+
+        elif attr == 'mismatch_num':
+            hsp.mistmatch_num = hsp.aln_span - hsp.ident_num - hsp.gap_num
+
+        elif attr == 'gapopen_num':
+            if not hasattr(hsp, 'query') or not hasattr(hsp, 'hit'):
+                raise AttributeError
+            hsp.gapopen_num = _compute_gapopen_num(hsp)
 
     # if the attr is a percent value, calculate it
     if attr == 'ident_pct':
-        func2 = lambda hsp: hsp.ident_num / float(hsp.aln_span) * 100
-    elif attr == 'pos_pct':
-        func = lambda hsp: hsp.pos_num / float(hsp.aln_span) * 100
-    elif attr == 'gap_pct':
-        func2 = lambda hsp: hsp.gap_num / float(hsp.aln_span) * 100
-    else:
-        func2 = None
+        hsp.ident_pct = hsp.ident_num / float(hsp.aln_span) * 100
 
-    # set the pct values
-    if func2 is not None:
-        value = func2(hsp)
-        setattr(hsp, attr, value)
+    elif attr == 'pos_pct':
+        hsp.pos_pct = hsp.pos_num / float(hsp.aln_span) * 100
+
+    elif attr == 'gap_pct':
+        hsp.gap_pct = hsp.gap_num / float(hsp.aln_span) * 100
 
 
 class BlastTabParser(object):
@@ -209,6 +207,7 @@ class BlastTabParser(object):
         self.line = self.handle.readline().strip()
 
     def __iter__(self):
+        """Iterate over BlastTabParser, yields query results."""
         # stop iteration if file has no lines
         if not self.line:
             return
@@ -216,6 +215,10 @@ class BlastTabParser(object):
         elif self.has_comments:
             iterfunc = self._parse_commented_qresult
         else:
+            if self.line.startswith("#"):
+                raise ValueError("Encountered unexpected character '#' at the"
+                                 " beginning of a line. Set comments=True if"
+                                 " the file is a commented file.")
             iterfunc = self._parse_qresult
 
         for qresult in iterfunc():
@@ -240,7 +243,7 @@ class BlastTabParser(object):
         return fields
 
     def _parse_commented_qresult(self):
-        """Yield `QueryResult` objects from a commented file (PRIVATE)."""
+        """Yield ``QueryResult`` objects from a commented file (PRIVATE)."""
         while True:
             comments = self._parse_comments()
             if comments:
@@ -315,8 +318,9 @@ class BlastTabParser(object):
         """Return a dictionary of parsed row values (PRIVATE)."""
         fields = self.fields
         columns = self.line.strip().split('\t')
-        assert len(fields) == len(columns), "Expected %i columns, found: " \
-            "%i" % (len(fields), len(columns))
+        if len(fields) != len(columns):
+            raise ValueError("Expected %i columns, found: "
+                             "%i" % (len(fields), len(columns)))
 
         qresult, hit, hsp, frag = {}, {}, {}, {}
         for idx, value in enumerate(columns):
@@ -421,21 +425,21 @@ class BlastTabParser(object):
                     for seq_type in ('query', 'hit'):
                         if attr == seq_type + '_start':
                             value = min(value,
-                                    prev['frag'][seq_type + '_end']) - 1
+                                        prev['frag'][seq_type + '_end']) - 1
                         elif attr == seq_type + '_end':
                             value = max(value,
-                                    prev['frag'][seq_type + '_start'])
+                                        prev['frag'][seq_type + '_start'])
                     setattr(frag, attr, value)
                 # strand and frame setattr require the full parsed values
                 # to be set first
                 for seq_type in ('hit', 'query'):
                     # try to set hit and query frame
                     frame = self._get_frag_frame(frag, seq_type,
-                            prev['frag'])
+                                                 prev['frag'])
                     setattr(frag, '%s_frame' % seq_type, frame)
                     # try to set hit and query strand
                     strand = self._get_frag_strand(frag, seq_type,
-                            prev['frag'])
+                                                   prev['frag'])
                     setattr(frag, '%s_strand' % seq_type, strand)
 
                 hsp = HSP([frag])
@@ -472,7 +476,7 @@ class BlastTabParser(object):
     def _get_frag_frame(self, frag, seq_type, parsedict):
         """Return fragment frame for given object (PRIVATE).
 
-        Returns `HSPFragment` frame given the object, its sequence type,
+        Returns ``HSPFragment`` frame given the object, its sequence type,
         and its parsed dictionary values.
         """
         assert seq_type in ('query', 'hit')
@@ -489,7 +493,7 @@ class BlastTabParser(object):
     def _get_frag_strand(self, frag, seq_type, parsedict):
         """Return fragment strand for given object (PRIVATE).
 
-        Returns `HSPFragment` strand given the object, its sequence type,
+        Returns ``HSPFragment`` strand given the object, its sequence type,
         and its parsed dictionary values.
         """
         # NOTE: this will never set the strands as 0 for protein
@@ -529,7 +533,8 @@ class BlastTabIndexer(SearchIndexer):
                 self._key_idx = fields.index('qaccver')
             else:
                 raise ValueError("Custom fields is missing an ID column. "
-                        "One of these must be present: 'qseqid', 'qacc', or 'qaccver'.")
+                                 "One of these must be present: 'qseqid', "
+                                 "'qacc', or 'qaccver'.")
 
     def __iter__(self):
         """Iterate over the file handle; yields key, start offset, and length."""
@@ -703,7 +708,7 @@ class BlastTabWriter(object):
 
     def _build_rows(self, qresult):
         """Return a string containing tabular rows of the QueryResult object (PRIVATE)."""
-        coordinates = set(['qstart', 'qend', 'sstart', 'send'])
+        coordinates = {'qstart', 'qend', 'sstart', 'send'}
         qresult_lines = ''
         for hit in qresult:
             for hsp in hit:
@@ -758,7 +763,7 @@ class BlastTabWriter(object):
         strand = getattr(hsp, '%s_strand' % seq_type, None)
         if strand is None:
             raise ValueError("Required attribute %r not found." %
-                    ('%s_strand' % (seq_type)))
+                             ('%s_strand' % (seq_type)))
         # switch start <--> end coordinates if strand is -1
         if strand < 0:
             if field.endswith('start'):
@@ -820,7 +825,7 @@ class BlastTabWriter(object):
 
         # list into ';'-delimited string
         elif field in ('sallseqid', 'sallacc', 'staxids', 'sscinames',
-                'scomnames', 'sblastnames', 'sskingdoms'):
+                       'scomnames', 'sblastnames', 'sskingdoms'):
             value = ';'.join(value)
 
         # everything else
@@ -834,7 +839,7 @@ class BlastTabWriter(object):
         comments = []
         # inverse mapping of the long-short name map, required
         # for writing comments
-        inv_field_map = dict((v, k) for k, v in _LONG_SHORT_MAP.items())
+        inv_field_map = {v: k for k, v in _LONG_SHORT_MAP.items()}
 
         # try to anticipate qress without version
         if not hasattr(qres, 'version'):
