@@ -33,6 +33,7 @@ class CodonAlphabet(Alphabet):
 
 
 def get_codon_alphabet(codon_table, gap_char="-"):
+    """Construct a CodonAlphabet from a given codon table."""
     letters = list(codon_table.forward_table.keys())
     letters.extend(codon_table.stop_codons)
     letters.extend(codon_table.start_codons)
@@ -48,6 +49,20 @@ def get_codon_alphabet(codon_table, gap_char="-"):
 default_codon_alphabet = get_codon_alphabet(default_codon_table)
 
 
+def compare_codon_alphabet(codon_alphabet_1, codon_alphabet_2):
+    """Compare two codon alphabet.
+
+    Return True is they have the same letter and gap_char,
+    Otherwise return False.
+    """
+    for codon_alphabet in [codon_alphabet_1, codon_alphabet_2]:
+        if not isinstance(codon_alphabet, CodonAlphabet):
+            raise TypeError("Only CodonAlphabet object accepted."
+                            "{} is provided.".format(object(codon_alphabet)))
+    return (set(codon_alphabet_1.letters) == set(codon_alphabet_2.letters) and
+            codon_alphabet_1.gap_char == codon_alphabet_2.gap_char)
+
+
 if __name__ == "__main__":
     from Bio._utils import run_doctest
     run_doctest()
