@@ -5,11 +5,8 @@
 
 """Parsing TRANSFAC files."""
 
-import warnings
 
-from Bio import BiopythonParserWarning
 from Bio import motifs
-from Bio.Alphabet import IUPAC
 
 
 class Motif(motifs.Motif, dict):
@@ -63,10 +60,10 @@ class Motif(motifs.Motif, dict):
     For more information, see the TRANSFAC documentation.
     """
 
-    multiple_value_keys = set(['BF', 'OV', 'HP', 'BS', 'HC', 'DT', 'DR'])
+    multiple_value_keys = {'BF', 'OV', 'HP', 'BS', 'HC', 'DT', 'DR'}
     # These keys can occur multiple times for one motif
 
-    reference_keys = set(['RX', 'RA', 'RT', 'RL'])
+    reference_keys = {'RX', 'RA', 'RT', 'RL'}
     # These keys occur for references
 
 
@@ -90,7 +87,7 @@ class Record(list):
         return write(self)
 
 
-def read(handle):
+def read(handle, strict=True):
     """Parse a transfac format handle into a Record object."""
     annotations = {}
     references = []
@@ -102,14 +99,18 @@ def read(handle):
             continue
         key_value = line.split(None, 1)
         key = key_value[0].strip()
-        assert len(key) == 2, 'The key value of a TRANSFAC motif line should have 2 characters: "{0:s}"'.format(line)
+        if strict:
+            if len(key) != 2:
+                raise ValueError('The key value of a TRANSFAC motif line '
+                                 'should have 2 characters: '
+                                 '"{0:s}"'.format(line))
         if len(key_value) == 2:
             value = key_value[1].strip()
-            if not line.partition('  ')[1]:
-                warnings.warn(
-                    'A TRANSFAC motif line should have 2 spaces between key and value columns: "{0:s}"'.format(line),
-                    BiopythonParserWarning
-                )
+            if strict:
+                if not line.partition('  ')[1]:
+                    raise ValueError('A TRANSFAC motif line should have 2 '
+                                     'spaces between key and value columns: '
+                                     '"{0:s}"'.format(line))
         if key == 'VV':
             record.version = value
         elif key in ('P0', 'PO'):  # Old TRANSFAC files use PO instead of P0
@@ -127,32 +128,38 @@ def read(handle):
                 key = key_value[0].strip()
                 if len(key_value) == 2:
                     value = key_value[1].strip()
-                    if not line.partition('  ')[1]:
-                        warnings.warn(
-                            'A TRANSFAC motif line should have 2 spaces between key and value columns: "{0:s}"'.format(
-                                line),
-                            BiopythonParserWarning)
+                    if strict:
+                        if not line.partition('  ')[1]:
+                            raise ValueError('A TRANSFAC motif line should '
+                                             'have 2 spaces between key and '
+                                             'value columns: '
+                                             '"{0:s}"'.format(line))
                 try:
                     i = int(key)
                 except ValueError:
                     break
                 if length == 0 and i == 0:
-                    warnings.warn(
-                        'A TRANSFAC matrix should start with "01" as first row of the matrix, '
-                        'but this matrix uses "00": "{0:s}"'.format(line),
-                        BiopythonParserWarning)
+                    if strict:
+                        raise ValueError('A TRANSFAC matrix should start with '
+                                         '"01" as first row of the matrix, '
+                                         'but this matrix uses "00": '
+                                         '"{0:s}"'.format(line))
                 else:
                     length += 1
                 if i != length:
                     raise ValueError('The TRANSFAC matrix row number does not '
                                      'match the position in the matrix: '
                                      '"{0:s}"'.format(line))
-                if len(key) == 1:
-                    warnings.warn(
-                        'A TRANSFAC matrix line should have a 2 digit key at the start of the lin ("{0:02d}"), '
-                        'but this matrix uses "{0:d}": "{1:s}".'.format(i, line),
-                        BiopythonParserWarning)
-                assert len(key_value) == 2, 'A TRANSFAC matrix line should have a key and a value: "{0:s}"'.format(line)
+                if strict:
+                    if len(key) == 1:
+                        raise ValueError('A TRANSFAC matrix line should have a '
+                                         '2 digit key at the start of the line '
+                                         '("{0:02d}"), but this matrix uses '
+                                         '"{0:d}": "{1:s}".'.format(i, line))
+                    if len(key_value) != 2:
+                        raise ValueError('A TRANSFAC matrix line should have '
+                                         'a key and a value: '
+                                         '"{0:s}"'.format(line))
                 values = value.split()[:4]
                 if len(values) != 4:
                     raise ValueError('A TRANSFAC matrix line should have a '
@@ -181,7 +188,7 @@ def read(handle):
             references.append(reference)
         elif key == '//':
             if counts is not None:
-                motif = Motif(alphabet=IUPAC.unambiguous_dna, counts=counts)
+                motif = Motif(alphabet='ACGT', counts=counts)
                 motif.update(annotations)
                 motif.references = references
                 record.append(motif)
@@ -241,7 +248,7 @@ XX
                     if length == 0:
                         continue
                     sequence = motif.degenerate_consensus
-                    letters = sorted(motif.alphabet.letters)
+                    letters = sorted(motif.alphabet)
                     line = "      ".join(["P0"] + letters)
 
                     lines.append(line)
